/* $Id: AuthorityGroup.java 1535200 2013-10-23 22:23:21Z kwright $ */

/**
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements. See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.apache.manifoldcf.authorities.authgroups;

import org.apache.manifoldcf.core.interfaces.*;
import org.apache.manifoldcf.authorities.interfaces.*;
import java.util.*;

/** This is the implementation of the authority group interface, which describes a paper object
* to be manipulated in order to create, edit, or save an authority group definition.
*/
public class AuthorityGroup implements IAuthorityGroup
{
  public static final String _rcsid = "@(#)$Id: AuthorityGroup.java 1535200 2013-10-23 22:23:21Z kwright $";

  // data
  protected boolean isNew = true;
  protected String name = null;
  protected String description = null;

  /** Constructor.
  */
  public AuthorityGroup()
  {
  }

  /** Clone this object.
  *@return the cloned object.
  */
  public AuthorityGroup duplicate()
  {
    AuthorityGroup rval = new AuthorityGroup();
    rval.isNew = isNew;
    rval.name = name;
    rval.description = description;
    return rval;
  }

  /** Set 'isnew' condition.
  *@param isnew true if this is a new instance.
  */
  public void setIsNew(boolean isnew)
  {
    this.isNew = isnew;
  }
  
  /** Get 'isnew' condition.
  *@return true if this is a new connection, false otherwise.
  */
  public boolean getIsNew()
  {
    return isNew;
  }

  /** Set name.
  *@param name is the name.
  */
  public void setName(String name)
  {
    this.name = name;
  }

  /** Get name.
  *@return the name
  */
  public String getName()
  {
    return name;
  }

  /** Set description.
  *@param description is the description.
  */
  public void setDescription(String description)
  {
    this.description = description;
  }

  /** Get description.
  *@return the description
  */
  public String getDescription()
  {
    return description;
  }

}
